from sktran_disco.tests.common import default_atmosphere, default_geometry
import sktran_disco.interface as do
import sasktran as sk
import xarray as xr
import numpy as np


def test_simple_calculation():
    atmo = default_atmosphere()
    geo = default_geometry()

    wavelengths = np.arange(280, 800, 1)

    engine = do.EngineDO(geometry=geo, atmosphere=atmo, wavelengths=wavelengths)

    rad = engine.calculate_radiance(output_format='xarray')


def test_multiple_lines_of_sight():
    atmo = default_atmosphere()
    geo = default_geometry()

    wavelengths = np.arange(280, 800, 10)

    engine = do.EngineDO(geometry=geo, atmosphere=atmo, wavelengths=wavelengths)

    rad = engine.calculate_radiance(output_format='xarray')

    los_rads = []
    for los in geo.lines_of_sight:
        new_geo = sk.Geometry()
        new_geo.lines_of_sight = [los]
        new_geo.sun = geo.sun

        engine = do.EngineDO(geometry=new_geo, atmosphere=atmo, wavelengths=wavelengths)

        los_rads.append(engine.calculate_radiance(output_format='xarray').isel(los=0))

    los_rads = xr.concat(los_rads, dim='los')
    diff = rad - los_rads

    np.testing.assert_array_equal(diff['radiance'].values, 0)


def test_num_streams():
    atmo = default_atmosphere()
    geo = default_geometry(mjd=54732, zenith=0, azimuth=0)

    wavelengths = [360]
    all_rad =[]
    for num_stream in np.arange(2, 41, 2):
        engine = do.EngineDO(geometry=geo, atmosphere=atmo, wavelengths=wavelengths)
        engine.num_streams = num_stream

        all_rad.append(engine.calculate_radiance(output_format='xarray').isel(los=0, wavelength=0))

    all_rad = xr.concat(all_rad, dim='streams')

    true_values = np.array([0.10616244, 0.11598831, 0.11630997, 0.11622187, 0.11619075,
                            0.11618959, 0.11619159, 0.11619204, 0.11619181, 0.1161916 ,
                            0.11619153, 0.11619153, 0.11619153, 0.11619153, 0.11619153,
                            0.11619152, 0.11619152, 0.11619152, 0.11619152, 0.11619152])

    np.testing.assert_array_almost_equal(all_rad['radiance'].values, true_values)


def test_num_layers():
    atmo = default_atmosphere()
    geo = default_geometry(mjd=54732, zenith=0, azimuth=0)

    wavelengths = [360]
    all_rad =[]
    for num_layer in [1, 2, 10, 50, 100]:
        engine = do.EngineDO(geometry=geo, atmosphere=atmo, wavelengths=wavelengths)
        engine.num_layers = num_layer

        all_rad.append(engine.calculate_radiance(output_format='xarray').isel(los=0, wavelength=0))

    all_rad = xr.concat(all_rad, dim='streams')

    true_values = np.array([0.11622926, 0.11620927, 0.11619428, 0.11619204, 0.11619162])

    np.testing.assert_array_almost_equal(all_rad['radiance'].values, true_values)
